@@ -402,7 +402,7 @@ def test_private_computed_field():
     class MyModel(BaseModel):
         x: int
 
-        @computed_field
+        @computed_field(repr=True)
         def _double(self) -> int:
             return self.x * 2
 
@@ -506,6 +506,7 @@ def test_abstractmethod():
     assert m.model_dump() == {'side': 4.0, 'area': 5.0}
 
 
+@pytest.mark.skipif(sys.version_info < (3, 12), reason='error message is different on older versions')
 @pytest.mark.parametrize(
     'bases',
     [
@@ -527,7 +528,9 @@ def test_abstractmethod_missing(bases: Tuple[Any, ...]):
     class Square(AbstractSquare):
         pass
 
-    with pytest.raises(TypeError, match="Can't instantiate abstract class Square with abstract methods? area"):
+    with pytest.raises(
+        TypeError, match="Can't instantiate abstract class Square without an implementation for abstract method 'area'"
+    ):
         Square(side=4.0)
 
 
@@ -741,3 +744,16 @@ def test_generic_computed_field():
 
     with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
         B[int]().model_dump()
+
+
+def test_computed_field_override_raises():
+    class Model(BaseModel):
+        name: str = 'foo'
+
+    with pytest.raises(ValueError, match="you can't override a field with a computed field"):
+
+        class SubModel(Model):
+            @computed_field
+            @property
+            def name(self) -> str:
+                return 'bar'
