@@ -32,7 +32,18 @@ from typing import (
 import pytest
 from dirty_equals import HasRepr, IsStr
 from pydantic_core import CoreSchema, core_schema
-from typing_extensions import Annotated, Literal, OrderedDict, ParamSpec, TypeVarTuple, Unpack, get_args
+from typing_extensions import (
+    Annotated,
+    Literal,
+    OrderedDict,
+    ParamSpec,
+    TypeVarTuple,
+    Unpack,
+    get_args,
+)
+from typing_extensions import (
+    TypeVar as TypingExtensionsTypeVar,
+)
 
 from pydantic import (
     BaseModel,
@@ -2622,3 +2633,140 @@ def test_reverse_order_generic_hashability():
     m1 = Model[int](x=1)
     m2 = Model[int](x=1)
     assert len({m1, m2}) == 1
+
+
+def test_serialize_unsubstituted_typevars_bound() -> None:
+    class ErrorDetails(BaseModel):
+        foo: str
+
+    ErrorDataT = TypeVar('ErrorDataT', bound=ErrorDetails)
+
+    class Error(BaseModel, Generic[ErrorDataT]):
+        message: str
+        details: ErrorDataT
+
+    class MyErrorDetails(ErrorDetails):
+        bar: str
+
+    sample_error = Error(
+        message='We just had an error',
+        details=MyErrorDetails(foo='var', bar='baz'),
+    )
+    assert sample_error.details.model_dump() == {
+        'foo': 'var',
+        'bar': 'baz',
+    }
+    assert sample_error.model_dump() == {
+        'message': 'We just had an error',
+        'details': {
+            'foo': 'var',
+            'bar': 'baz',
+        },
+    }
+
+    sample_error = Error[ErrorDetails](
+        message='We just had an error',
+        details=MyErrorDetails(foo='var', bar='baz'),
+    )
+    assert sample_error.details.model_dump() == {
+        'foo': 'var',
+        'bar': 'baz',
+    }
+    assert sample_error.model_dump() == {
+        'message': 'We just had an error',
+        'details': {
+            'foo': 'var',
+        },
+    }
+
+    sample_error = Error[MyErrorDetails](
+        message='We just had an error',
+        details=MyErrorDetails(foo='var', bar='baz'),
+    )
+    assert sample_error.details.model_dump() == {
+        'foo': 'var',
+        'bar': 'baz',
+    }
+    assert sample_error.model_dump() == {
+        'message': 'We just had an error',
+        'details': {
+            'foo': 'var',
+            'bar': 'baz',
+        },
+    }
+
+
+@pytest.mark.parametrize(
+    'type_var',
+    [
+        TypingExtensionsTypeVar('ErrorDataT', default=BaseModel),
+        TypeVar('ErrorDataT', BaseModel, str),
+    ],
+    ids=['default', 'constraint'],
+)
+def test_serialize_unsubstituted_typevars_bound(
+    type_var: Type[BaseModel],
+) -> None:
+    class ErrorDetails(BaseModel):
+        foo: str
+
+    class Error(BaseModel, Generic[type_var]):  # type: ignore
+        message: str
+        details: type_var
+
+    class MyErrorDetails(ErrorDetails):
+        bar: str
+
+    sample_error = Error(
+        message='We just had an error',
+        details=MyErrorDetails(foo='var', bar='baz'),
+    )
+    assert sample_error.details.model_dump() == {
+        'foo': 'var',
+        'bar': 'baz',
+    }
+    assert sample_error.model_dump() == {
+        'message': 'We just had an error',
+        'details': {},
+    }
+
+    sample_error = Error[ErrorDetails](
+        message='We just had an error',
+        details=MyErrorDetails(foo='var', bar='baz'),
+    )
+    assert sample_error.details.model_dump() == {
+        'foo': 'var',
+        'bar': 'baz',
+    }
+    assert sample_error.model_dump() == {
+        'message': 'We just had an error',
+        'details': {
+            'foo': 'var',
+        },
+    }
+
+    sample_error = Error[MyErrorDetails](
+        message='We just had an error',
+        details=MyErrorDetails(foo='var', bar='baz'),
+    )
+    assert sample_error.details.model_dump() == {
+        'foo': 'var',
+        'bar': 'baz',
+    }
+    assert sample_error.model_dump() == {
+        'message': 'We just had an error',
+        'details': {
+            'foo': 'var',
+            'bar': 'baz',
+        },
+    }
+
+
+def test_mix_default_and_constraints() -> None:
+    T = TypingExtensionsTypeVar('T', str, int, default=str)
+
+    msg = 'Pydantic does not support mixing more than one of TypeVar bounds, constraints and defaults'
+    with pytest.raises(NotImplementedError, match=msg):
+
+        class _(BaseModel, Generic[T]):
+            x: T
